@@ -14,12 +14,19 @@ pub use super::syntax::*;
 pub(super) use super::paste;
 pub(super) use super::paste::{IdentFrag, IdentFragInfallible};
 
+pub(super) use general_context::{
+    DriverlessContext, GeneralContext, GeneralContextBuf, MissingContextError,
+};
+
 /// Context during expansion
 ///
 /// References the driver, and digested information about it.
 /// Also represents where in the driver we are,
 /// including repetition context.
-#[derive(Debug, Clone)]
+//
+// Has to be Copy because otherwise the borrow checker doesn't understand
+// what's going on with ctx_buf in Template::expand.
+#[derive(Debug, Clone, Copy)]
 pub struct Context<'c> {
     pub top: &'c syn::DeriveInput,
     pub template_crate: &'c syn::Path,
@@ -29,9 +36,18 @@ pub struct Context<'c> {
     pub field: Option<&'c WithinField<'c>>,
     pub within_loop: WithinLoop,
     pub pvariants: &'c [PreprocessedVariant<'c>],
-    pub definitions: Definitions<'c>,
+    pub defs: DefinitionsContext<'c>,
+}
+
+/// Part of the context relating to user-defined expansions and conditions
+#[derive(Debug, Clone, Copy, Default)]
+pub struct DefinitionsContext<'c> {
+    pub defs: Definitions<'c>,
     pub nesting_depth: u16,
-    pub nesting_parent: Option<(&'c Context<'c>, &'c DefinitionName)>,
+    pub nesting_parent: Option<(
+        &'c DefinitionsContext<'c>, //
+        &'c DefinitionName,
+    )>,
 }
 
 #[derive(Debug)]
@@ -52,6 +68,7 @@ pub struct WithinVariant<'c> {
     pub fields: &'c syn::Fields,
     pub pmetas: &'c meta::PreprocessedMetas,
     pub pfields: &'c [PreprocessedField],
+    pub index: u32,
 }
 
 #[derive(Debug, Clone)]
@@ -86,7 +103,7 @@ pub struct Definitions<'c> {
 }
 
 /// Special processing instructions returned by
-/// [`special_before_element_hook`](SubstParseContext::special_before_element_hook)
+/// [`before_element_hook`](SpecialParseContext::before_element_hook)
 pub enum SpecialInstructions {
     /// This template is finished
     ///
@@ -125,8 +142,12 @@ pub enum SpecialInstructions {
 pub trait SubstParseContext: Sized {
     /// Uninhabited iff this lexical context is within `${paste }`
     type NotInPaste: Debug + Copy + Sized;
+    /// Uninhabited iff this lexical context is within `${concat }`
+    type NotInConcat: Debug + Copy + Sized;
     /// Uninhabited iff this lexical context is within a condition.
     type NotInBool: Debug + Copy + Sized;
+    /// Uninhabited unless this lexical context is within `${concat }`
+    type ConcatOnly: Debug + Copy + Sized;
     /// Uninhabited unless this lexical context is within a condition.
     type BoolOnly: Debug + Copy + Sized;
 
@@ -141,11 +162,15 @@ pub trait SubstParseContext: Sized {
     /// This has to be in this trait because
     /// `${dbg }` contains a `Template` but `dbg(...)` contains a `Subst`.
     ///
+    /// For all `ExpansionContext` impls, should be `Template<Self>`.
+    // We'd use an associated type default, but they're unstable.
+    ///
     /// We make bespoke output for each context; for boolean this is sui
     /// generis, and for expansions it's in [`ExpansionOutput::dbg_expand`].
     type DbgContent: Parse + Debug + AnalyseRepeat + meta::FindRecogMetas;
 
     fn not_in_paste(span: &impl Spanned) -> syn::Result<Self::NotInPaste>;
+    fn not_in_concat(span: &impl Spanned) -> syn::Result<Self::NotInConcat>;
     fn not_in_bool(span: &impl Spanned) -> syn::Result<Self::NotInBool>;
 
     fn bool_only(span: &impl Spanned) -> syn::Result<Self::BoolOnly> {
@@ -153,33 +178,29 @@ pub trait SubstParseContext: Sized {
             "derive-deftly keyword is a condition - not valid as an expansion",
         ))
     }
+    fn concat_only(span: &impl Spanned) -> syn::Result<Self::ConcatOnly> {
+        Err(span
+            .error("derive-deftly keyword is only allowed within ${concat }"))
+    }
 
     /// When we find a `fmeta` etc. in this context, does it allow a value?
     ///
     /// Used by the template-scanning code, to report whether an `Xmeta`
     /// in the template justifies a value-bearing `Xmeta` attribute
     /// on/in the driver, or just a boolean.
-    fn meta_recog_usage() -> meta::Usage;
+    fn meta_recog_usage(m: &meta::SubstMeta<Self>) -> meta::Usage {
+        if m.default.is_some() {
+            meta::Usage::VALUE_ONLY
+        } else {
+            meta::Usage::VALUE
+        }
+    }
 
     /// For communicating through `parse_special`
-    type SpecialParseContext: Default;
-
-    /// Handle any special syntax for a special kind of template context.
     ///
-    /// This method is called only when parsing multi-element [`Template`]s,
-    /// It's a hook, called before parsing each `TemplateElement`.
-    ///
-    /// It should consume any special syntax as appropriate,
-    ///
-    /// The default implementation is a no-op.
-    /// The only non-default implementation is in `paste.rs`, for `$<...>` -
-    /// see [`paste::AngleBrackets`].
-    fn special_before_element_hook(
-        _special: &mut Self::SpecialParseContext,
-        _input: ParseStream,
-    ) -> syn::Result<Option<SpecialInstructions>> {
-        Ok(None)
-    }
+    /// Used when parsing `${paste ..}` to handle
+    /// the special `>`-based end condition.
+    type SpecialParseContext: SpecialParseContext + Default;
 
     /// Parse using `f`, within parens in boolean context, not otherwise
     ///
@@ -220,6 +241,52 @@ pub trait SubstParseContext: Sized {
     }
 }
 
+pub trait SpecialParseContext {
+    /// Handle any special syntax for a special kind of template context.
+    ///
+    /// This method is called only when parsing multi-element [`Template`]s,
+    /// It's a hook, called before parsing each `TemplateElement`.
+    ///
+    /// It should consume any special syntax as appropriate,
+    ///
+    /// The default implementation is a no-op.
+    /// The only non-default implementation is in `paste.rs`, for `$<...>` -
+    /// see [`paste::AngleBrackets`].
+    fn before_element_hook(
+        &mut self,
+        _input: ParseStream,
+    ) -> syn::Result<Option<SpecialInstructions>>;
+}
+
+impl SpecialParseContext for () {
+    fn before_element_hook(
+        &mut self,
+        _input: ParseStream,
+    ) -> syn::Result<Option<SpecialInstructions>> {
+        Ok(None)
+    }
+}
+
+/// Inhabited iff this lexical content may expand to arbitrary tokens
+///
+/// Calculated automatically
+#[allow(type_alias_bounds)]
+pub type AllowTokens<O: SubstParseContext> = (
+    <O as SubstParseContext>::NotInPaste,
+    <O as SubstParseContext>::NotInConcat,
+);
+
+/// Extensions/helpers for `SubstParseContext`
+pub trait SubstParseContextExt: SubstParseContext {
+    fn allow_tokens(span: &impl Spanned) -> syn::Result<AllowTokens<Self>>;
+}
+impl<O: SubstParseContext> SubstParseContextExt for O {
+    fn allow_tokens(span: &impl Spanned) -> syn::Result<AllowTokens<Self>> {
+        let span = span.span();
+        Ok((O::not_in_paste(&span)?, (O::not_in_concat(&span)?)))
+    }
+}
+
 /// Expansion output accumulator, for a template lexical context
 ///
 /// Each template lexical context has a distinct type which
@@ -322,7 +389,7 @@ pub trait ExpansionOutput: SubstParseContext {
     /// during template parsing.
     fn append_tokens_with(
         &mut self,
-        np: &Self::NotInPaste,
+        allow_tokens: &AllowTokens<Self>,
         f: impl FnOnce(&mut TokenAccumulator) -> syn::Result<()>,
     ) -> syn::Result<()>;
 
@@ -361,10 +428,10 @@ pub trait ExpansionOutput: SubstParseContext {
     // rather badly and had other warts.
     fn append_tokens(
         &mut self,
-        np: &Self::NotInPaste,
+        allow_tokens: &AllowTokens<Self>,
         tokens: impl ToTokens,
     ) -> syn::Result<()> {
-        self.append_tokens_with(np, |out| {
+        self.append_tokens_with(allow_tokens, |out| {
             out.append(tokens);
             Ok(())
         })
@@ -376,7 +443,9 @@ pub trait ExpansionOutput: SubstParseContext {
     /// This one is used for special cases, eg `${ignore ...}`
     fn new_with_span(kw_span: Span) -> Self;
 
-    fn default_subst_meta_as(kw: Span) -> syn::Result<meta::SubstAs<Self>>;
+    fn default_subst_meta_as(kw: Span) -> syn::Result<meta::SubstAs<Self>> {
+        Err(kw.error("missing `as ...` in meta expansion"))
+    }
 
     /// Implement the core of the `ignore` keyword
     ///
@@ -391,14 +460,15 @@ pub trait ExpansionOutput: SubstParseContext {
     ///   * Without a trailing newline
     ///   * Subsume it into `self`
     ///
-    /// Failures are to be reported, and subsumed into `self`.
-    fn dbg_expand(
+    /// Expansion errors are to be reported, and subsumed into `self`.
+    /// Failures to write may be thrown.
+    fn dbg_expand<'c>(
         &mut self,
         kw_span: Span,
-        ctx: &Context,
+        ctx: GeneralContext<'c>,
         msg: &mut String,
         content: &Self::DbgContent,
-    );
+    ) -> fmt::Result;
 }
 
 /// Convenience trait providing `item.expand()`
@@ -412,7 +482,11 @@ pub trait ExpansionOutput: SubstParseContext {
 /// It also avoids having to make all of these expansion methods
 /// members of the `ExpansionOutput` trait.
 pub trait Expand<O> {
-    fn expand(&self, ctx: &Context, out: &mut O) -> syn::Result<()>;
+    fn expand<'c>(
+        &self,
+        ctx: GeneralContext<'c>,
+        out: &mut O,
+    ) -> syn::Result<()>;
 }
 /// Convenience trait providing `fn expand(self)`, infallible version
 ///
@@ -420,7 +494,7 @@ pub trait Expand<O> {
 /// within the output accumulator
 /// and therefore do not need to return them.
 pub trait ExpandInfallible<O> {
-    fn expand(&self, ctx: &Context, out: &mut O);
+    fn expand<'c>(&self, ctx: GeneralContext<'c>, out: &mut O);
 }
 
 /// Accumulates tokens, or errors
@@ -492,19 +566,21 @@ impl TokenAccumulator {
 
 impl SubstParseContext for TokenAccumulator {
     type NotInPaste = ();
+    type NotInConcat = ();
     type NotInBool = ();
-    type DbgContent = Template<TokenAccumulator>;
-    fn not_in_bool(_: &impl Spanned) -> syn::Result<()> {
-        Ok(())
-    }
+    type DbgContent = Template<Self>;
     fn not_in_paste(_: &impl Spanned) -> syn::Result<()> {
         Ok(())
     }
-    fn meta_recog_usage() -> meta::Usage {
-        meta::Usage::Value
+    fn not_in_concat(_: &impl Spanned) -> syn::Result<()> {
+        Ok(())
+    }
+    fn not_in_bool(_: &impl Spanned) -> syn::Result<()> {
+        Ok(())
     }
 
     type BoolOnly = Void;
+    type ConcatOnly = Void;
 
     type SpecialParseContext = ();
 }
@@ -545,7 +621,7 @@ impl ExpansionOutput for TokenAccumulator {
         };
 
         match inner.tokens() {
-            Ok(ts) => self.append(grouping.surround(ts)),
+            Ok(ts) => self.append(grouping.surround(ident.span(), ts)),
             Err(e) => self.record_error(e),
         }
         Ok(())
@@ -559,11 +635,11 @@ impl ExpansionOutput for TokenAccumulator {
         ty: syn::Type,
         grouping: Grouping,
     ) {
-        self.append(grouping.surround(ty));
+        self.append(grouping.surround(ty.span(), ty));
     }
     fn append_tokens_with(
         &mut self,
-        _not_in_paste: &(),
+        (_not_in_paste, _not_in_concat): &((), ()),
         f: impl FnOnce(&mut TokenAccumulator) -> syn::Result<()>,
     ) -> syn::Result<()> {
         f(self)
@@ -585,32 +661,28 @@ impl ExpansionOutput for TokenAccumulator {
         Self::new()
     }
 
-    fn default_subst_meta_as(kw: Span) -> syn::Result<meta::SubstAs<Self>> {
-        Err(kw.error("missing `as ...` in meta expansion"))
-    }
-
     fn ignore_impl(self) -> syn::Result<()> {
         self.0.map(|_: TokenStream| ())
     }
 
-    fn dbg_expand(
+    fn dbg_expand<'c>(
         &mut self,
         _kw_span: Span,
-        ctx: &Context,
+        ctx: GeneralContext<'c>,
         msg: &mut String,
         content: &Template<TokenAccumulator>,
-    ) {
+    ) -> fmt::Result {
         let mut child = TokenAccumulator::new();
         content.expand(ctx, &mut child);
         let child = child.tokens();
         match &child {
-            Err(e) => write!(msg, "/* ERROR: {} */", e),
-            Ok(y) => write!(msg, "{}", y),
+            Err(e) => write!(msg, "/* ERROR: {} */", e)?,
+            Ok(y) => write!(msg, "{}", y)?,
         }
-        .expect("write! failed");
         match child {
             Ok(y) => self.append(y),
             Err(e) => self.record_error(e),
         }
+        Ok(())
     }
 }
