@@ -9,7 +9,8 @@ use super::framework::*;
 /// and collects the identifier fragments being pasted.
 #[derive(Debug)]
 pub struct Items {
-    tspan: Span,
+    /// Error reporting for bad identifiers, and also the output span
+    span: Span,
     items: Vec<Item>,
     errors: Vec<syn::Error>,
     atoms: Vec<AtomForReport>,
@@ -139,6 +140,42 @@ impl<T: ToTokens + quote::IdentFragment + Display> IdentFrag for T {
     }
 }
 
+//---------- restricted case conversions ----------
+
+/// Token that an associated `ChangeCase` has been checked for legal context
+///
+/// Really this ought to be a field in each variant of `ChangeCase<O>`
+/// but that would make lots of pointless generics.
+///
+/// The keyword-specific part of the check is just
+/// an avoidance of user error, anyway.
+#[derive(Debug)]
+pub struct ChangeCaseOkIn<O: SubstParseContext> {
+    pub not_in_bool: O::NotInBool,
+    #[allow(dead_code)]
+    paste_context_checked: (),
+}
+
+impl<O: SubstParseContext> ChangeCaseOkIn<O> {
+    pub fn new_checked(case: ChangeCase, span: Span) -> syn::Result<Self> {
+        let not_in_bool = O::not_in_bool(&span)?;
+
+        let paste_context_checked = case
+            .ok_in_paste()
+            // Otherwise, it's a ${concat }-only case change keyword (& beta)
+            .or_else(|()| {
+                let _: O::ConcatOnly = O::concat_only(&span)?;
+                let _: beta::Enabled = beta::Enabled::new_for_syntax(span)?;
+                Ok::<_, syn::Error>(())
+            })?;
+
+        Ok(ChangeCaseOkIn {
+            not_in_bool,
+            paste_context_checked,
+        })
+    }
+}
+
 //---------- case conversion ----------
 
 /// Define cases using heck
@@ -154,23 +191,35 @@ impl<T: ToTokens + quote::IdentFragment + Display> IdentFrag for T {
 /// API.  It will become the variant name in `ChangeCase`.
 ///
 /// `$keyword` are the keywords we recognise for this case conversion.
-#[cfg(feature = "case")]
 macro_rules! define_cases { {
     $(
-        $heck:ident $( $keyword:literal )*,
+        $heck:ident $( $keyword:literal )* $in_paste:ident,
     )*
 } => {
     #[derive(Debug, Clone, Copy)]
     pub enum ChangeCase {
-        $( $heck, )*
+      $(
+        #[cfg(feature = "case")]
+        $heck,
+      )*
     }
 
     impl FromStr for ChangeCase {
         type Err = ();
+        #[allow(unreachable_code)]
         fn from_str(s: &str) -> Result<Self, ()> {
             Ok(match s {
               $(
-                $( $keyword )|* => ChangeCase::$heck,
+                $( $keyword )|* => {
+                    // Minimal build, so no need to bother with a proper error.
+                    #[cfg(not(feature = "case"))]
+                    panic!(
+ "case changing not supported, enable `case` feature of `derive-deftly`"
+                    );
+
+                    #[cfg(feature = "case")]
+                    ChangeCase::$heck
+                },
               )*
                 _ => return Err(()),
             })
@@ -178,54 +227,36 @@ macro_rules! define_cases { {
     }
 
     impl ChangeCase {
-        fn apply(self, input: &str) -> String {
+        fn apply(self, #[allow(unused)] input: &str) -> String {
             match self {
               $(
+                #[cfg(feature = "case")]
                 ChangeCase::$heck => heck::$heck(input).to_string(),
               )*
             }
         }
-    }
-} }
 
-#[cfg(not(feature = "case"))]
-macro_rules! define_cases { {
-    $(
-        $heck:ident $( $keyword:literal )*,
-    )*
-} => {
-    #[derive(Debug, Clone, Copy)]
-    pub enum ChangeCase {}
-
-    impl FromStr for ChangeCase {
-        type Err = ();
-        fn from_str(s: &str) -> Result<Self, ()> {
-            match s {
+        fn ok_in_paste(self) -> Result<(), ()> {
+            match self {
               $(
-                $( $keyword )|* => {}
+                #[cfg(feature = "case")]
+                ChangeCase::$heck => $in_paste(()),
               )*
-                _ => return Err(()),
             }
-            // Minimal build, so no need to bother with a proper error.
-            panic!(
- "case changing not supported, enable `case` feature of `derive-deftly`"
-            );
-        }
-    }
-
-    impl ChangeCase {
-        fn apply(self, _input: &str) -> String {
-            match self {}
         }
     }
 } }
 
 define_cases! {
     // heck API       our keyword (and aliases)  our example(s)
-    AsUpperCamelCase    "pascal_case"         "upper_camel_case"   ,
-    AsLowerCamelCase    "lower_camel_case"                         ,
-    AsSnakeCase         "snake_case"                               ,
-    AsShoutySnakeCase   "shouty_snake_case"                        ,
+    AsUpperCamelCase    "pascal_case"         "upper_camel_case"   Ok,
+    AsLowerCamelCase    "lower_camel_case"                         Ok,
+    AsSnakeCase         "snake_case"                               Ok,
+    AsShoutySnakeCase   "shouty_snake_case"                        Ok,
+    AsKebabCase         "kebab_case"                               Err,
+    AsShoutyKebabCase   "shouty_kebab_case"                        Err,
+    AsTitleCase         "title_case"                               Err,
+    AsTrainCase         "train_case"                               Err,
 }
 
 //---------- TokenPastesAsIdent ----------
@@ -261,9 +292,9 @@ impl<T: ToTokens> IdentFrag for TokenPastesAsIdent<T> {
 //---------- implementation ----------
 
 impl Items {
-    pub fn new(tspan: Span) -> Self {
+    pub fn new(span: Span) -> Self {
         Items {
-            tspan,
+            span,
             items: vec![],
             errors: vec![],
             atoms: vec![],
@@ -329,7 +360,7 @@ use AssemblyInstruction as AI;
 /// Any surrounding path elements, generics, etc., of a nontrivial
 /// expansion, are handled by `assemble`, not here.
 fn mk_ident<'i>(
-    out_span: Span,
+    span: Span,
     change_case: Option<ChangeCase>,
     pieces: impl Iterator<Item = Piece<'i>> + Clone,
     atoms: Vec<AtomForReport>,
@@ -340,11 +371,7 @@ fn mk_ident<'i>(
     } else {
         whole
     };
-    Pasted {
-        span: out_span,
-        whole,
-        atoms,
-    }
+    Pasted { span, whole, atoms }
 }
 
 /// Error that stands in for the `syn::Error` from an invalid identifier
@@ -500,13 +527,13 @@ fn ident_from_str() {
     chk_err("0");
 }
 
-pub fn expand(
-    ctx: &Context<'_>,
-    kw_span: Span,
+pub fn expand<'c>(
+    ctx: GeneralContext<'c>,
+    tspan: Span,
     content: &Template<paste::Items>,
     out: &mut impl ExpansionOutput,
 ) -> syn::Result<()> {
-    let mut items = paste::Items::new(kw_span);
+    let mut items = paste::Items::new(tspan);
     content.expand(ctx, &mut items);
     items.assemble(out, None)
 }
@@ -573,7 +600,7 @@ impl Items {
         }
 
         match Self::assemble_inner(
-            self.tspan,
+            self.span,
             self.items,
             change_case,
             self.atoms,
@@ -648,7 +675,9 @@ impl Items {
             })?
             .map(|(pos, _)| pos);
 
-        fn plain_strs(items: &[Item]) -> impl Iterator<Item = Piece> + Clone {
+        fn plain_strs(
+            items: &[Item],
+        ) -> impl Iterator<Item = Piece<'_>> + Clone {
             items.iter().map(|item| match item {
                 Item::Plain { text, span } => (text.as_str(), *span),
                 _ => panic!("non plain item"),
@@ -708,30 +737,34 @@ impl Items {
 
 impl SubstParseContext for Items {
     type NotInPaste = Void;
+    type NotInConcat = ();
     type NotInBool = ();
     type BoolOnly = Void;
-    type DbgContent = Template<Items>;
+    type ConcatOnly = Void;
+    type DbgContent = Template<Self>;
 
     fn not_in_bool(_: &impl Spanned) -> syn::Result<()> {
         Ok(())
     }
+    fn not_in_concat(_: &impl Spanned) -> syn::Result<()> {
+        Ok(())
+    }
 
     fn not_in_paste(span: &impl Spanned) -> syn::Result<Void> {
         Err(span
             .error("not allowed in within ${paste ...} (or case_changing)"))
     }
-    fn meta_recog_usage() -> meta::Usage {
-        meta::Usage::Value
-    }
 
     type SpecialParseContext = Option<AngleBrackets>;
+}
 
-    fn special_before_element_hook(
-        special: &mut Option<AngleBrackets>,
+impl SpecialParseContext for Option<AngleBrackets> {
+    fn before_element_hook(
+        &mut self,
         input: ParseStream,
     ) -> syn::Result<Option<SpecialInstructions>> {
         if input.peek(Token![>]) {
-            if let Some(state) = special {
+            if let Some(state) = self {
                 let _: Token![>] = input.parse()?;
                 state.found_close = true;
                 return Ok(Some(SpecialInstructions::EndOfTemplate));
@@ -859,7 +892,7 @@ impl ExpansionOutput for Items {
     }
     fn append_tokens_with(
         &mut self,
-        not_in_paste: &Void,
+        (not_in_paste, _not_in_compat): &(Void, ()),
         _: impl FnOnce(&mut TokenAccumulator) -> syn::Result<()>,
     ) -> syn::Result<()> {
         void::unreachable(*not_in_paste)
@@ -883,7 +916,7 @@ impl ExpansionOutput for Items {
 
     fn ignore_impl(self) -> syn::Result<()> {
         let Items {
-            tspan: _,
+            span: _,
             items: _,
             atoms: _,
             errors,
@@ -895,64 +928,65 @@ impl ExpansionOutput for Items {
         accum.finish()
     }
 
-    fn dbg_expand(
+    fn dbg_expand<'c>(
         &mut self,
         kw_span: Span,
-        ctx: &Context,
+        ctx: GeneralContext<'c>,
         msg: &mut String,
         content: &Template<Items>,
-    ) {
+    ) -> fmt::Result {
         let mut child = Items::new(kw_span);
         content.expand(ctx, &mut child);
         let Items {
-            tspan: _,
+            span: _,
             items,
             errors,
             atoms: _,
         } = child;
 
-        (|| {
-            for e in &errors {
-                writeln!(msg, "ERROR: {}", e)?;
-            }
-            for (sep, i) in izip!(
-                chain!([""], iter::repeat(" ")), //
-                &items,
-            ) {
-                write!(msg, "{}", sep)?;
-                match i {
-                    Item::Plain { text, .. } => write!(msg, "{:?}", text)?,
-                    Item::Complex { surround, text, .. } => {
-                        write!(
-                            msg,
-                            "{:?}+{:?}+{:?}",
-                            surround.pre.to_string(),
-                            text,
-                            surround.post.to_string(),
-                        )?;
-                    }
+        for e in &errors {
+            writeln!(msg, "ERROR: {}", e)?;
+        }
+        for (sep, i) in izip!(
+            chain!([""], iter::repeat(" ")), //
+            &items,
+        ) {
+            write!(msg, "{}", sep)?;
+            match i {
+                Item::Plain { text, .. } => write!(msg, "{:?}", text)?,
+                Item::Complex { surround, text, .. } => {
+                    write!(
+                        msg,
+                        "{:?}+{:?}+{:?}",
+                        surround.pre.to_string(),
+                        text,
+                        surround.post.to_string(),
+                    )?;
                 }
             }
-            Ok::<_, fmt::Error>(())
-        })()
-        .expect("write! failed");
+        }
 
         self.items.extend(items);
         self.errors.extend(errors);
+        Ok(())
     }
 }
 
 impl Expand<Items> for TemplateElement<Items> {
-    fn expand(&self, ctx: &Context, out: &mut Items) -> syn::Result<()> {
+    fn expand<'c>(
+        &self,
+        ctx: GeneralContext<'c>,
+        out: &mut Items,
+    ) -> syn::Result<()> {
         match self {
-            TE::Ident(ident) => out.append_identfrag_toks(&ident)?,
+            TE::Ident(ident, ..) => out.append_identfrag_toks(&ident)?,
             TE::LitStr(lit) => out.append_syn_litstr(&lit),
             TE::Subst(e) => e.expand(ctx, out)?,
             TE::Repeat(e) => e.expand(ctx, out),
-            TE::Literal(_, not_in_paste)
-            | TE::Punct(_, not_in_paste)
-            | TE::Group { not_in_paste, .. } => {
-                void::unreachable(*not_in_paste)
+            TE::Literal(_, allow_tokens)
+            | TE::Punct(_, allow_tokens)
+            | TE::Group { allow_tokens, .. } => {
+                void::unreachable(allow_tokens.0)
             }
         }
         Ok(())
