@@ -3,6 +3,37 @@
 use super::prelude::*;
 use proc_macro_crate::{crate_name, FoundCrate};
 
+//---------- macros for defining impls ----------
+
+macro_rules! impl_deref { { $ttype:ty, $fname:tt, $ftype:ty } => {
+    impl Deref for $ttype {
+        type Target = $ftype;
+        fn deref(&self) -> &$ftype {
+            &self.$fname
+        }
+    }
+} }
+macro_rules! impl_to_tokens { {
+    $ttype:ident $( [ $($tgen:tt)* ] )?,
+    $fname:tt
+} => {
+    impl $( < $($tgen)* > )? ToTokens for $ttype $( < $($tgen)* > )? {
+        fn to_tokens(&self, out: &mut TokenStream) {
+            self.$fname.to_tokens(out)
+        }
+    }
+} }
+macro_rules! impl_display { {
+    $ttype:ident $( [ $($tgen:tt)* ] )?,
+    $fname:tt
+} => {
+    impl $( < $($tgen)* > )? Display for $ttype $( < $($tgen)* > )? {
+        fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+            Display::fmt(&self.$fname, f)
+        }
+    }
+} }
+
 //---------- dbg dumps of macro inputs/outputs ----------
 
 /// Calls eprintln! but only if enabled
@@ -74,6 +105,12 @@ pub mod dprint {
 
 //---------- misc ----------
 
+pub type ErrorGenerator<'c> = &'c (dyn Fn(Span) -> syn::Error + 'c);
+
+macro_rules! error_generator { { $msg:expr } => {
+    (&|span: Span| span.error($msg)) as ErrorGenerator
+} }
+
 /// Construct a braced group from a token expansion
 ///
 /// Calls `f` to expand tokens, and returns a braced group containing
@@ -92,9 +129,7 @@ pub fn delimit_token_group(
 ) -> syn::Result<proc_macro2::Group> {
     let mut out = TokenAccumulator::default();
     f(&mut out)?;
-    let out = out.tokens()?;
-    let mut out = proc_macro2::Group::new(delim, out);
-    out.set_span(delim_span);
+    let out = group_new_with_span(delim, delim_span, out.tokens()?);
     Ok(out)
 }
 
@@ -124,6 +159,12 @@ impl<T: Parse> Parse for Concatenated<T> {
     }
 }
 
+impl<T: ToTokens> Concatenated<T> {
+    pub fn is_empty(&self) -> bool {
+        self.0.is_empty()
+    }
+}
+
 impl<T: ToTokens> ToTokens for Concatenated<T> {
     fn to_tokens(&self, out: &mut TokenStream) {
         for item in &self.0 {
@@ -132,6 +173,56 @@ impl<T: ToTokens> ToTokens for Concatenated<T> {
     }
 }
 
+/// Returns a `Group`, with a specified span
+pub fn group_new_with_span(
+    delimiter: Delimiter,
+    span: Span,
+    ts: TokenStream,
+) -> proc_macro2::Group {
+    let mut g_out = proc_macro2::Group::new(delimiter, ts);
+    g_out.set_span(span);
+    g_out
+}
+
+/// Returns a `Group` that is like `g_in` but with stream `ts`
+pub fn group_clone_set_stream(
+    g_in: &proc_macro2::Group,
+    ts: TokenStream,
+) -> proc_macro2::Group {
+    group_new_with_span(
+        g_in.delimiter(),
+        // We lose some span information here, sadly.
+        g_in.span(),
+        ts,
+    )
+}
+
+/// Respan the resolution hygiene of all identifiers to span
+pub fn respan_hygiene(input: TokenStream, span: Span) -> TokenStream {
+    /// Recursive call.  Separate so we can wrap it, etc.
+    pub fn recurse(input: TokenStream, span: Span) -> TokenStream {
+        let map_span = |tok_span: Span| tok_span.resolved_at(span);
+
+        input
+            .into_iter()
+            .update(|tt| match tt {
+                TT::Group(g) => {
+                    let new_span = map_span(g.span());
+                    *g = proc_macro2::Group::new(
+                        g.delimiter(),
+                        recurse(g.stream(), span),
+                    );
+                    g.set_span(new_span);
+                }
+                TT::Ident(i) => i.set_span(map_span(i.span())),
+                TT::Punct(p) => p.set_span(map_span(p.span())),
+                TT::Literal(l) => l.set_span(map_span(l.span())),
+            })
+            .collect()
+    }
+    recurse(input, span)
+}
+
 /// Add any missing colons before `< >`'d generic arguments
 ///
 /// ```rust,ignore
@@ -319,6 +410,11 @@ impl ErrorAccumulator {
         self.defused = true;
         self.bad.take()
     }
+
+    /// If there were any errors, examine them (as a combined `syn::Error`)
+    pub fn examine(&self) -> Option<&syn::Error> {
+        self.bad.as_ref()
+    }
 }
 
 impl Drop for ErrorAccumulator {
@@ -377,14 +473,33 @@ pub enum Grouping {
 }
 
 impl Grouping {
-    pub fn surround(&self, ts: impl ToTokens) -> TokenStream {
+    /// Surround `ts` as specified by this `Grouping`
+    ///
+    /// `none_span` is the span to use for `None`-delimiters.
+    /// It should generally be the principal span of the contained thing,
+    /// whatever that is.  For a type, that's the primary ident.
+    pub fn surround(&self, none_span: Span, ts: impl ToTokens) -> TokenStream {
         let ts = ts.to_token_stream();
         match self {
             Grouping::Ungrouped => ts,
             Grouping::Invisible => {
-                proc_macro2::Group::new(Delimiter::None, ts).to_token_stream()
+                let mut g = proc_macro2::Group::new(Delimiter::None, ts);
+                g.set_span(none_span);
+                g.to_token_stream()
             }
             Grouping::Parens => {
+                // We must use call site span (the default for Group::new
+                // since if we surround a type with `( )` with the same
+                // span as its contents, Rust generates an "unnecessary parens"
+                // warning.  We add actual `( )` mostly in ${Xmeta as ty}.
+                // We also *replace* any `( )` which are in the outside
+                // of a type in `TokenAccumulator::append_syn_type`.
+                //
+                // When we do that we mangle the original span.  That may
+                // produces worse errors if there is an error, but it avoids
+                // needless warnings.  This ought to be very rare.
+                // (but it does happen in tests/expand/idpaste.rs, which
+                // deliberately has this weird construct.)
                 proc_macro2::Group::new(Delimiter::Parenthesis, ts)
                     .to_token_stream()
             }
@@ -392,6 +507,50 @@ impl Grouping {
     }
 }
 
+//---------- DocAttribute ----------
+
+/// `syn::Attribute`(s) (zero or more) that are known to be a `#[doc ]`.
+#[derive(Debug, Clone)]
+pub struct DocAttributes(Concatenated<syn::Attribute>);
+
+impl Parse for DocAttributes {
+    fn parse(input: ParseStream) -> syn::Result<Self> {
+        let attrs = input.call(syn::Attribute::parse_outer)?;
+        for attr in &attrs {
+            if !attr.path().is_ident("doc") {
+                return Err(attr
+                    .path()
+                    .error("only doc attributes are supported"));
+            }
+        }
+        Ok(DocAttributes(Concatenated(attrs)))
+    }
+}
+
+impl Deref for DocAttributes {
+    type Target = Concatenated<syn::Attribute>;
+    fn deref(&self) -> &Concatenated<syn::Attribute> {
+        &self.0
+    }
+}
+impl DocAttributes {
+    pub fn to_tokens_with_addendum(
+        self,
+        addendum_text: fmt::Arguments<'_>,
+    ) -> TokenStream {
+        let addendum = if self.0.is_empty() {
+            None
+        } else {
+            Some(addendum_text.to_string())
+        };
+        let mut out = self.0.to_token_stream();
+        if let Some(addendum) = addendum {
+            out.extend(quote!( #[doc = #addendum] ))
+        }
+        out
+    }
+}
+
 //---------- IdentAny ----------
 
 /// Like `syn::Ident` but parses using `parse_any`, accepting keywords
@@ -402,22 +561,14 @@ impl Grouping {
 /// Needs care when used with user data, since it might be a keyword,
 /// in which case it's not really an *identifier*.
 pub struct IdentAny(pub syn::Ident);
+impl_deref!(IdentAny, 0, syn::Ident);
+impl_to_tokens!(IdentAny, 0);
+
 impl Parse for IdentAny {
     fn parse(input: ParseStream) -> syn::Result<Self> {
         Ok(IdentAny(Ident::parse_any(input)?))
     }
 }
-impl Deref for IdentAny {
-    type Target = syn::Ident;
-    fn deref(&self) -> &syn::Ident {
-        &self.0
-    }
-}
-impl ToTokens for IdentAny {
-    fn to_tokens(&self, out: &mut TokenStream) {
-        self.0.to_tokens(out)
-    }
-}
 impl<T: AsRef<str> + ?Sized> PartialEq<T> for IdentAny {
     fn eq(&self, rhs: &T) -> bool {
         self.0.eq(rhs)
@@ -485,55 +636,92 @@ impl<'t, 'o> TokenOutputTrimmer<'t, 'o> {
 
 //---------- TemplateName ----------
 
-#[derive(Debug, Clone)]
-pub struct TemplateName(syn::Ident);
+pub type TemplateName = SyntheticMacroName<SyntheticMacroTemplate>;
+pub type ModuleName = SyntheticMacroName<SyntheticMacroModule>;
 
-impl TemplateName {
+// Generic arrangements
+
+/// Name of a thing of kind `Kind`
+#[derive(Debug, Clone)]
+pub struct SyntheticMacroName<Kind>(syn::Ident, Kind);
+
+/// Kind marker types
+pub trait SyntheticMacroKind: Default + Copy + Debug {
+    const WHAT: &'static str;
+}
+
+// Kinds
+
+#[derive(Debug, Clone, Copy, Default)]
+pub struct SyntheticMacroTemplate;
+impl SyntheticMacroKind for SyntheticMacroTemplate {
+    const WHAT: &'static str = "template";
+}
+
+#[derive(Debug, Clone, Copy, Default)]
+pub struct SyntheticMacroModule;
+impl SyntheticMacroKind for SyntheticMacroModule {
+    const WHAT: &'static str = "module";
+}
+
+// Non-generic functions, with substantive code
+
+fn synthetic_macro_name_check(
+    ident: &syn::Ident,
+    what: &str,
+) -> syn::Result<()> {
+    let s = ident.to_string();
+    match s.chars().find(|&c| c != '_') {
+        None => Err(format!(
+            "{} name cannot consist entirely of underscores",
+            what,
+        )),
+        Some(c) => {
+            if c.is_lowercase() {
+                Err(format!(
+ "{} name may not start with a lowercase letter (after any underscores)", what,
+                ))
+            } else {
+                Ok(())
+            }
+        }
+    }
+    .map_err(|emsg| ident.error(emsg))
+}
+
+fn synthetic_macro_name_macro_name(
+    ident: &syn::Ident,
+    what: &str,
+) -> syn::Ident {
+    format_ident!("derive_deftly_{}_{}", what, ident)
+}
+
+// Impls, generic
+
+impl_to_tokens!(SyntheticMacroName[K], 0);
+impl_display!(SyntheticMacroName[K], 0);
+
+impl<K: SyntheticMacroKind> SyntheticMacroName<K> {
     pub fn macro_name(&self) -> syn::Ident {
-        format_ident!("derive_deftly_template_{}", &self.0)
+        synthetic_macro_name_macro_name(&self.0, K::WHAT)
     }
 }
 
-impl Parse for TemplateName {
+impl<K: SyntheticMacroKind> Parse for SyntheticMacroName<K> {
     fn parse(input: ParseStream) -> syn::Result<Self> {
         let ident: syn::Ident = input.parse()?;
         ident.try_into()
     }
 }
-impl TryFrom<syn::Ident> for TemplateName {
-    type Error = syn::Error;
-    fn try_from(ident: syn::Ident) -> syn::Result<TemplateName> {
-        let s = ident.to_string();
-        match s.chars().find(|&c| c != '_') {
-            None => {
-                Err("template name cannot consist entirely of underscores")
-            }
-            Some(c) => {
-                if c.is_lowercase() {
-                    Err(
- "template name may not start with a lowercase letter (after any underscores)"
-                )
-                } else {
-                    Ok(())
-                }
-            }
-        }
-        .map_err(|emsg| ident.error(emsg))?;
 
-        Ok(TemplateName(ident))
+impl<K: SyntheticMacroKind> TryFrom<syn::Ident> for SyntheticMacroName<K> {
+    type Error = syn::Error;
+    fn try_from(ident: syn::Ident) -> syn::Result<SyntheticMacroName<K>> {
+        synthetic_macro_name_check(&ident, K::WHAT)?;
+        Ok(SyntheticMacroName(ident, K::default()))
     }
 }
-impl ToTokens for TemplateName {
-    fn to_tokens(&self, out: &mut TokenStream) {
-        self.0.to_tokens(out)
-    }
-}
-impl Display for TemplateName {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        Display::fmt(&self.0, f)
-    }
-}
-impl quote::IdentFragment for TemplateName {
+impl<K> quote::IdentFragment for SyntheticMacroName<K> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         quote::IdentFragment::fmt(&self.0, f)
     }
@@ -542,6 +730,55 @@ impl quote::IdentFragment for TemplateName {
     }
 }
 
+//---------- TemplatePath ----------
+
+pub type TemplatePath = SyntheticMacroPath<SyntheticMacroTemplate>;
+pub type ModulePath = SyntheticMacroPath<SyntheticMacroModule>;
+
+// Invariant: is valid (so, last segment is a valid TemplateName)
+#[derive(Debug, Clone)]
+pub struct SyntheticMacroPath<Kind>(syn::Path, Kind);
+
+impl_to_tokens!(SyntheticMacroPath[K], 0);
+
+fn synthetic_macro_path_parse(
+    input: ParseStream,
+    what: &str,
+) -> syn::Result<syn::Path> {
+    let path = syn::Path::parse_mod_style(input)?;
+
+    let last = path.segments.last().ok_or_else(|| {
+        path.leading_colon
+            .as_ref()
+            .expect("path with no tokens!")
+            .error("empty path not allowed here")
+    })?;
+
+    synthetic_macro_name_check(&last.ident, what)?;
+
+    Ok(path)
+}
+
+fn synthetic_macro_path_macro_path(path: &syn::Path, what: &str) -> syn::Path {
+    let mut out = path.clone();
+    let last = &mut out.segments.last_mut().expect("became empty!").ident;
+    *last = synthetic_macro_name_macro_name(last, what);
+    out
+}
+
+impl<K: SyntheticMacroKind> Parse for SyntheticMacroPath<K> {
+    fn parse(input: ParseStream) -> syn::Result<Self> {
+        let path = synthetic_macro_path_parse(input, K::WHAT)?;
+        Ok(SyntheticMacroPath(path, K::default()))
+    }
+}
+
+impl<K: SyntheticMacroKind> SyntheticMacroPath<K> {
+    pub fn macro_path(&self) -> syn::Path {
+        synthetic_macro_path_macro_path(&self.0, K::WHAT)
+    }
+}
+
 //---------- engine_macro_name ----------
 
 /// Return a full path to the location of `derive_deftly_engine`.
