@@ -45,4 +45,29 @@ derive_deftly_adhoc! {
     struct $<ViaDefine $VIA_DEFINE>;
 }
 
+derive_deftly_adhoc! {
+    DataType beta_deftly:
+    ${kebab_case $tname}
+}
+derive_deftly_adhoc! {
+    DataType beta_deftly:
+    $<${kebab_case $tname}>
+}
+
+derive_deftly_adhoc! {
+    DataType beta_deftly:
+
+    fn unspanned_unimpl() {
+      $(
+        let _: $ftype = (42).$<into>();
+      )
+    }
+
+    fn spanned_unimpl() {
+      $(
+        let _: $ftype = (42).${paste_spanned $fname into}();
+      )
+    }
+}
+
 fn main() {}
